@@ -20,16 +20,24 @@
 OpenStack Client interface. Handles the REST calls and responses.
 """
 
-import errno
+import copy
 import functools
 import glob
 import hashlib
+import imp
+import itertools
 import logging
 import os
-import time
+import pkgutil
+import re
+import warnings
 
+from keystoneauth1 import adapter
+from keystoneauth1 import session
+from oslo_utils import importutils
+from oslo_utils import netutils
+import pkg_resources
 import requests
-from requests import adapters
 
 try:
     import json
@@ -38,14 +46,13 @@ except ImportError:
 
 from six.moves.urllib import parse
 
+from novaclient import api_versions
 from novaclient import exceptions
-from novaclient.openstack.common.gettextutils import _
-from novaclient.openstack.common import network_utils
+from novaclient import extension as ext
+from novaclient.i18n import _, _LW
 from novaclient import service_catalog
 from novaclient import utils
 
-SENSITIVE_HEADERS = ('X-Auth-Token',)
-
 
 class _ClientConnectionPool(object):
 
@@ -53,150 +60,53 @@ class _ClientConnectionPool(object):
         self._adapters = {}
 
     def get(self, url):
-        """
-        Store and reuse HTTP adapters per Service URL.
-        """
+        """Store and reuse HTTP adapters per Service URL."""
         if url not in self._adapters:
-            self._adapters[url] = adapters.HTTPAdapter()
+            self._adapters[url] = session.TCPKeepAliveAdapter()
 
         return self._adapters[url]
 
 
-class CompletionCache(object):
-    """The completion cache is how we support tab-completion with novaclient.
+class SessionClient(adapter.LegacyJsonAdapter):
 
-    The `Manager` writes object IDs and Human-IDs to the completion-cache on
-    object-show, object-list, and object-create calls.
-
-    The `nova.bash_completion` script then uses these files to provide the
-    actual tab-completion.
-
-    The cache directory layout is:
-
-        ~/.novaclient/
-            <hash-of-endpoint-and-username>/
-                <resource>-id-cache
-                <resource>-human-id-cache
-    """
-    def __init__(self, username, auth_url, attributes=('id', 'human_id')):
-        self.directory = self._make_directory_name(username, auth_url)
-        self.attributes = attributes
-
-    def _make_directory_name(self, username, auth_url):
-        """Creates a unique directory name based on the auth_url and username
-        of the current user.
-        """
-        uniqifier = hashlib.md5(username.encode('utf-8') +
-                                auth_url.encode('utf-8')).hexdigest()
-        base_dir = utils.env('NOVACLIENT_UUID_CACHE_DIR',
-                             default="~/.novaclient")
-        return os.path.expanduser(os.path.join(base_dir, uniqifier))
-
-    def _prepare_directory(self):
-        try:
-            os.makedirs(self.directory, 0o755)
-        except OSError:
-            # NOTE(kiall): This is typically either permission denied while
-            #              attempting to create the directory, or the
-            #              directory already exists. Either way, don't
-            #              fail.
-            pass
-
-    def clear_class(self, obj_class):
-        self._prepare_directory()
-
-        resource = obj_class.__name__.lower()
-        resource_glob = os.path.join(self.directory, "%s-*-cache" % resource)
-
-        for filename in glob.iglob(resource_glob):
-            try:
-                os.unlink(filename)
-            except OSError as e:
-                if e.errno != errno.ENOENT:
-                    raise
-
-    def _write_attribute(self, resource, attribute, value):
-        self._prepare_directory()
-
-        filename = "%s-%s-cache" % (resource, attribute.replace('_', '-'))
-        path = os.path.join(self.directory, filename)
-
-        with open(path, 'a') as f:
-            f.write("%s\n" % value)
-
-    def write_object(self, obj):
-        resource = obj.__class__.__name__.lower()
-
-        for attribute in self.attributes:
-            value = getattr(obj, attribute, None)
-            if value:
-                self._write_attribute(resource, attribute, value)
-
-
-class SessionClient(object):
-
-    def __init__(self, session, auth, interface, service_type, region_name):
-        self.session = session
-        self.auth = auth
-
-        self.interface = interface
-        self.service_type = service_type
-        self.region_name = region_name
+    def __init__(self, *args, **kwargs):
+        self.times = []
+        self.timings = kwargs.pop('timings', False)
+        self.api_version = kwargs.pop('api_version', None)
+        self.api_version = self.api_version or api_versions.APIVersion()
+        super(SessionClient, self).__init__(*args, **kwargs)
 
     def request(self, url, method, **kwargs):
-        kwargs.setdefault('user_agent', 'python-novaclient')
-        kwargs.setdefault('auth', self.auth)
-        kwargs.setdefault('authenticated', False)
-
-        try:
-            kwargs['json'] = kwargs.pop('body')
-        except KeyError:
-            pass
-
-        headers = kwargs.setdefault('headers', {})
-        headers.setdefault('Accept', 'application/json')
-
-        endpoint_filter = kwargs.setdefault('endpoint_filter', {})
-        endpoint_filter.setdefault('interface', self.interface)
-        endpoint_filter.setdefault('service_type', self.service_type)
-        endpoint_filter.setdefault('region_name', self.region_name)
-
-        resp = self.session.request(url, method, raise_exc=False, **kwargs)
-
-        body = None
-        if resp.text:
-            try:
-                body = resp.json()
-            except ValueError:
-                pass
-
-        if resp.status_code >= 400:
+        kwargs.setdefault('headers', kwargs.get('headers', {}))
+        api_versions.update_headers(kwargs["headers"], self.api_version)
+        # NOTE(jamielennox): The standard call raises errors from
+        # keystoneauth1, where we need to raise the novaclient errors.
+        raise_exc = kwargs.pop('raise_exc', True)
+        with utils.record_time(self.times, self.timings, method, url):
+            resp, body = super(SessionClient, self).request(url,
+                                                            method,
+                                                            raise_exc=False,
+                                                            **kwargs)
+        # TODO(andreykurilin): uncomment this line, when we will be able to
+        #   check only nova-related calls
+        # api_versions.check_headers(resp, self.api_version)
+        if raise_exc and resp.status_code >= 400:
             raise exceptions.from_response(resp, body, url, method)
 
         return resp, body
 
-    def _cs_request(self, url, method, **kwargs):
-        # this function is mostly redundant but makes compatibility easier
-        kwargs.setdefault('authenticated', True)
-        return self.request(url, method, **kwargs)
+    def get_timings(self):
+        return self.times
 
-    def get(self, url, **kwargs):
-        return self._cs_request(url, 'GET', **kwargs)
-
-    def post(self, url, **kwargs):
-        return self._cs_request(url, 'POST', **kwargs)
-
-    def put(self, url, **kwargs):
-        return self._cs_request(url, 'PUT', **kwargs)
-
-    def delete(self, url, **kwargs):
-        return self._cs_request(url, 'DELETE', **kwargs)
+    def reset_timings(self):
+        self.times = []
 
 
 def _original_only(f):
-    """Indicates and enforces that this function can only be used if we are
-    using the original HTTPClient object.
+    """Decorator to indicate and enforce original HTTPClient object.
 
+    Indicates and enforces that this function can only be used if we are
+    using the original HTTPClient object.
     We use this to specify that if you use the newer Session HTTP client then
     you are aware that the way you use your client has been updated and certain
     functions are no longer allowed to be used.
@@ -226,15 +136,16 @@ class HTTPClient(object):
                  http_log_debug=False, auth_system='keystone',
                  auth_plugin=None, auth_token=None,
                  cacert=None, tenant_id=None, user_id=None,
-                 connection_pool=False):
+                 connection_pool=False, api_version=None):
         self.user = user
         self.user_id = user_id
         self.password = password
         self.projectid = projectid
         self.tenant_id = tenant_id
+        self.api_version = api_version or api_versions.APIVersion()
 
         self._connection_pool = (_ClientConnectionPool()
-                                if connection_pool else None)
+                                 if connection_pool else None)
 
         # This will be called by #_get_password if self.password is None.
         # EG if a password can only be obtained by prompting the user, but a
@@ -301,6 +212,10 @@ class HTTPClient(object):
                 # otherwise we will get all the requests logging messages
                 rql.setLevel(logging.WARNING)
 
+        self.service_catalog = None
+        self.services_url = {}
+        self.last_request_id = None
+
     def use_token_cache(self, use_it):
         self.os_cache = use_it
 
@@ -318,21 +233,47 @@ class HTTPClient(object):
     def reset_timings(self):
         self.times = []
 
-    def safe_header(self, name, value):
-        if name in SENSITIVE_HEADERS:
-            # because in python3 byte string handling is ... ug
-            v = value.encode('utf-8')
-            h = hashlib.sha1(v)
-            d = h.hexdigest()
-            return name, "{SHA1}%s" % d
-        else:
-            return name, value
+    def _redact(self, target, path, text=None):
+        """Replace the value of a key in `target`.
+
+        The key can be at the top level by specifying a list with a single
+        key as the path. Nested dictionaries are also supported by passing a
+        list of keys to be navigated to find the one that should be replaced.
+        In this case the last one is the one that will be replaced.
+
+        :param dict target: the dictionary that may have a key to be redacted;
+                            modified in place
+        :param list path: a list representing the nested structure in `target`
+                          that should be redacted; modified in place
+        :param string text: optional text to use as a replacement for the
+                            redacted key. if text is not specified, the
+                            default text will be sha1 hash of the value being
+                            redacted
+        """
+
+        key = path.pop()
+
+        # move to the most nested dict
+        for p in path:
+            try:
+                target = target[p]
+            except KeyError:
+                return
+
+        if key in target:
+            if text:
+                target[key] = text
+            elif target[key] is not None:
+                # because in python3 byte string handling is ... ug
+                value = target[key].encode('utf-8')
+                sha1sum = hashlib.sha1(value)
+                target[key] = "{SHA1}%s" % sha1sum.hexdigest()
 
     def http_log_req(self, method, url, kwargs):
         if not self.http_log_debug:
             return
 
-        string_parts = ['curl -i']
+        string_parts = ['curl -g -i']
 
         if not kwargs.get('verify', True):
             string_parts.append(' --insecure')
@@ -340,24 +281,38 @@ class HTTPClient(object):
         string_parts.append(" '%s'" % url)
         string_parts.append(' -X %s' % method)
 
+        headers = copy.deepcopy(kwargs['headers'])
+        self._redact(headers, ['X-Auth-Token'])
         # because dict ordering changes from 2 to 3
-        keys = sorted(kwargs['headers'].keys())
+        keys = sorted(headers.keys())
         for name in keys:
-            value = kwargs['headers'][name]
-            header = ' -H "%s: %s"' % self.safe_header(name, value)
+            value = headers[name]
+            header = ' -H "%s: %s"' % (name, value)
             string_parts.append(header)
 
         if 'data' in kwargs:
-            string_parts.append(" -d '%s'" % (kwargs['data']))
+            data = json.loads(kwargs['data'])
+            self._redact(data, ['auth', 'passwordCredentials', 'password'])
+            string_parts.append(" -d '%s'" % json.dumps(data))
         self._logger.debug("REQ: %s" % "".join(string_parts))
 
     def http_log_resp(self, resp):
         if not self.http_log_debug:
             return
+
+        if resp.text and resp.status_code != 400:
+            try:
+                body = json.loads(resp.text)
+                self._redact(body, ['access', 'token', 'id'])
+            except ValueError:
+                body = None
+        else:
+            body = None
+
         self._logger.debug("RESP: [%(status)s] %(headers)s\nRESP BODY: "
                            "%(text)s\n", {'status': resp.status_code,
                                           'headers': resp.headers,
-                                          'text': resp.text})
+                                          'text': json.dumps(body)})
 
     def open_session(self):
         if not self._connection_pool:
@@ -379,10 +334,10 @@ class HTTPClient(object):
                     self._session.close()
                 self._current_url = service_url
                 self._logger.debug(
-                        "New session created for: (%s)" % service_url)
+                    "New session created for: (%s)" % service_url)
                 self._session = requests.Session()
                 self._session.mount(service_url,
-                        self._connection_pool.get(service_url))
+                                    self._connection_pool.get(service_url))
             return self._session
         elif self._session:
             return self._session
@@ -395,6 +350,7 @@ class HTTPClient(object):
             kwargs['headers']['Content-Type'] = 'application/json'
             kwargs['data'] = json.dumps(kwargs['body'])
             del kwargs['body']
+        api_versions.update_headers(kwargs["headers"], self.api_version)
         if self.timeout is not None:
             kwargs.setdefault('timeout', self.timeout)
         kwargs['verify'] = self.verify_cert
@@ -411,6 +367,10 @@ class HTTPClient(object):
             url,
             **kwargs)
 
+        # TODO(andreykurilin): uncomment this line, when we will be able to
+        #   check only nova-related calls
+        # api_versions.check_headers(resp, self.api_version)
+
         self.http_log_resp(resp)
 
         if resp.text:
@@ -422,7 +382,7 @@ class HTTPClient(object):
             # or 'actively refused' in the body, so that's what we'll do.
             if resp.status_code == 400:
                 if ('Connection refused' in resp.text or
-                    'actively refused' in resp.text):
+                        'actively refused' in resp.text):
                     raise exceptions.ConnectionRefused(resp.text)
             try:
                 body = json.loads(resp.text)
@@ -431,21 +391,33 @@ class HTTPClient(object):
         else:
             body = None
 
+        self.last_request_id = (resp.headers.get('x-openstack-request-id')
+                                if resp.headers else None)
         if resp.status_code >= 400:
             raise exceptions.from_response(resp, body, url, method)
 
         return resp, body
 
     def _time_request(self, url, method, **kwargs):
-        start_time = time.time()
-        resp, body = self.request(url, method, **kwargs)
-        self.times.append(("%s %s" % (method, url),
-                           start_time, time.time()))
+        with utils.record_time(self.times, self.timings, method, url):
+            resp, body = self.request(url, method, **kwargs)
         return resp, body
 
     def _cs_request(self, url, method, **kwargs):
         if not self.management_url:
             self.authenticate()
+        if url is None:
+            # To get API version information, it is necessary to GET
+            # a nova endpoint directly without "v2/<tenant-id>".
+            magic_tuple = parse.urlsplit(self.management_url)
+            scheme, netloc, path, query, frag = magic_tuple
+            path = re.sub(r'v[1-9](\.[1-9][0-9]*)?/[a-z0-9]+$', '', path)
+            url = parse.urlunsplit((scheme, netloc, path, None, None))
+        else:
+            if self.service_catalog and not self.bypass_url:
+                url = self.get_service_url(self.service_type) + url
+            else:
+                url = self.management_url + url
 
         # Perform the request once. If we get a 401 back then it
         # might be because the auth token expired, so try to
@@ -455,8 +427,7 @@ class HTTPClient(object):
             if self.projectid:
                 kwargs['headers']['X-Auth-Project-Id'] = self.projectid
 
-            resp, body = self._time_request(self.management_url + url, method,
-                                            **kwargs)
+            resp, body = self._time_request(url, method, **kwargs)
             return resp, body
         except exceptions.Unauthorized as e:
             try:
@@ -467,8 +438,7 @@ class HTTPClient(object):
                 self.keyring_saved = False
                 self.authenticate()
                 kwargs['headers']['X-Auth-Token'] = self.auth_token
-                resp, body = self._time_request(self.management_url + url,
-                                                method, **kwargs)
+                resp, body = self._time_request(url, method, **kwargs)
                 return resp, body
             except exceptions.Unauthorized:
                 raise e
@@ -490,8 +460,23 @@ class HTTPClient(object):
     def delete(self, url, **kwargs):
         return self._cs_request(url, 'DELETE', **kwargs)
 
+    def get_service_url(self, service_type):
+        if service_type not in self.services_url:
+            url = self.service_catalog.url_for(
+                attr='region',
+                filter_value=self.region_name,
+                endpoint_type=self.endpoint_type,
+                service_type=service_type,
+                service_name=self.service_name,
+                volume_service_name=self.volume_service_name,)
+            url = url.rstrip('/')
+            self.services_url[service_type] = url
+        return self.services_url[service_type]
+
     def _extract_service_catalog(self, url, resp, body, extract_token=True):
-        """See what the auth service told us and process the response.
+        """Extract service catalog from input resource body.
+
+        See what the auth service told us and process the response.
         We may get redirected to another site, fail or actually get
         back a service catalog with a token and our endpoints.
         """
@@ -506,14 +491,7 @@ class HTTPClient(object):
                     self.auth_token = self.service_catalog.get_token()
                     self.tenant_id = self.service_catalog.get_tenant_id()
 
-                management_url = self.service_catalog.url_for(
-                    attr='region',
-                    filter_value=self.region_name,
-                    endpoint_type=self.endpoint_type,
-                    service_type=self.service_type,
-                    service_name=self.service_name,
-                    volume_service_name=self.volume_service_name,)
-                self.management_url = management_url.rstrip('/')
+                self.management_url = self.get_service_url(self.service_type)
                 return None
             except exceptions.AmbiguousEndpoints:
                 print(_("Found more than one valid endpoint. Use a more "
@@ -532,7 +510,9 @@ class HTTPClient(object):
             raise exceptions.from_response(resp, body, url)
 
     def _fetch_endpoints_from_auth(self, url):
-        """We have a token, but don't know the final endpoint for
+        """Fetch endpoint using token.
+
+        We have a token, but don't know the final endpoint for
         the region. We have to go back to the auth service and
         ask again. This request requires an admin-level token
         to work. The proxy token supplied could be from a low-level enduser.
@@ -553,7 +533,11 @@ class HTTPClient(object):
                                              extract_token=False)
 
     def authenticate(self):
-        magic_tuple = network_utils.urlsplit(self.auth_url)
+        if not self.auth_url:
+            msg = _("Authentication requires 'auth_url', which should be "
+                    "specified in '%s'") % self.__class__.__name__
+            raise exceptions.AuthorizationFailure(msg)
+        magic_tuple = netutils.urlsplit(self.auth_url)
         scheme, netloc, path, query, frag = magic_tuple
         port = magic_tuple.port
         if port is None:
@@ -615,9 +599,9 @@ class HTTPClient(object):
 
     def _save_keys(self):
         # Store the token/mgmt url in the keyring for later requests.
-        if (self.keyring_saver and self.os_cache and not self.keyring_saved
-                and self.auth_token and self.management_url
-                and self.tenant_id):
+        if (self.keyring_saver and self.os_cache and not self.keyring_saved and
+                self.auth_token and self.management_url and
+                self.tenant_id):
             self.keyring_saver.save(self.auth_token,
                                     self.management_url,
                                     self.tenant_id)
@@ -698,13 +682,21 @@ def _construct_http_client(username=None, password=None, project_id=None,
                            auth_system='keystone', auth_plugin=None,
                            auth_token=None, cacert=None, tenant_id=None,
                            user_id=None, connection_pool=False, session=None,
-                           auth=None):
+                           auth=None, user_agent='python-novaclient',
+                           interface=None, api_version=None, **kwargs):
+    # TODO(mordred): If not session, just make a Session, then return
+    # SessionClient always
     if session:
         return SessionClient(session=session,
                              auth=auth,
-                             interface=endpoint_type,
+                             interface=interface or endpoint_type,
                              service_type=service_type,
-                             region_name=region_name)
+                             region_name=region_name,
+                             service_name=service_name,
+                             user_agent=user_agent,
+                             timings=timings,
+                             api_version=api_version,
+                             **kwargs)
     else:
         # FIXME(jamielennox): username and password are now optional. Need
         # to test that they were provided in this mode.
@@ -731,26 +723,108 @@ def _construct_http_client(username=None, password=None, project_id=None,
                           os_cache=os_cache,
                           http_log_debug=http_log_debug,
                           cacert=cacert,
-                          connection_pool=connection_pool)
+                          connection_pool=connection_pool,
+                          api_version=api_version)
+
+
+def discover_extensions(version, only_contrib=False):
+    """Returns the list of extensions, which can be discovered by python path,
+    contrib path and by entry-point 'novaclient.extension'.
+
+    :param version: api version
+    :type version: str or novaclient.api_versions.APIVersion
+    :param only_contrib: search only in contrib directory or not
+    :type only_contrib: bool
+    """
+    if not isinstance(version, api_versions.APIVersion):
+        version = api_versions.get_api_version(version)
+    if only_contrib:
+        chain = _discover_via_contrib_path(version)
+    else:
+        chain = itertools.chain(_discover_via_python_path(),
+                                _discover_via_contrib_path(version),
+                                _discover_via_entry_points())
+    return [ext.Extension(name, module) for name, module in chain]
+
+
+def _discover_via_python_path():
+    for (module_loader, name, _ispkg) in pkgutil.iter_modules():
+        if name.endswith('_python_novaclient_ext'):
+            # NOTE(sdague): needed for python 2.x compatibility.
+            if not hasattr(module_loader, 'load_module'):
+                module_loader = module_loader.find_module(name)
+            module = module_loader.load_module(name)
+            if hasattr(module, 'extension_name'):
+                name = module.extension_name
+
+            yield name, module
+
+
+def _discover_via_contrib_path(version):
+    module_path = os.path.dirname(os.path.abspath(__file__))
+    ext_path = os.path.join(module_path, "v%s" % version.ver_major, 'contrib')
+    ext_glob = os.path.join(ext_path, "*.py")
+
+    for ext_path in glob.iglob(ext_glob):
+        name = os.path.basename(ext_path)[:-3]
+
+        if name == "__init__":
+            continue
+
+        module = imp.load_source(name, ext_path)
+        yield name, module
+
+
+def _discover_via_entry_points():
+    for ep in pkg_resources.iter_entry_points('novaclient.extension'):
+        name = ep.name
+        module = ep.load()
+
+        yield name, module
+
+
+def _get_client_class_and_version(version):
+    if not isinstance(version, api_versions.APIVersion):
+        version = api_versions.get_api_version(version)
+    else:
+        api_versions.check_major_version(version)
+    if version.is_latest():
+        raise exceptions.UnsupportedVersion(
+            _("The version should be explicit, not latest."))
+    return version, importutils.import_class(
+        "novaclient.v%s.client.Client" % version.ver_major)
 
 
 def get_client_class(version):
-    version_map = {
-        '1.1': 'novaclient.v1_1.client.Client',
-        '2': 'novaclient.v1_1.client.Client',
-        '3': 'novaclient.v3.client.Client',
-    }
-    try:
-        client_path = version_map[str(version)]
-    except (KeyError, ValueError):
-        msg = _("Invalid client version '%(version)s'. must be one of: "
-                "%(keys)s") % {'version': version,
-                               'keys': ', '.join(version_map.keys())}
-        raise exceptions.UnsupportedVersion(msg)
-
-    return utils.import_class(client_path)
+    """Returns Client class based on given version."""
+    warnings.warn(_LW("'get_client_class' is deprecated. "
+                      "Please use `novaclient.client.Client` instead."))
+    _api_version, client_class = _get_client_class_and_version(version)
+    return client_class
 
 
 def Client(version, *args, **kwargs):
-    client_class = get_client_class(version)
-    return client_class(*args, **kwargs)
+    """Initialize client object based on given version.
+
+    HOW-TO:
+    The simplest way to create a client instance is initialization with your
+    credentials::
+
+        >>> from novaclient import client
+        >>> nova = client.Client(VERSION, USERNAME, PASSWORD,
+        ...                      PROJECT_ID, AUTH_URL)
+
+    Here ``VERSION`` can be a string or
+    ``novaclient.api_versions.APIVersion`` obj. If you prefer string value,
+    you can use ``1.1`` (deprecated now), ``2`` or ``2.X``
+    (where X is a microversion).
+
+
+    Alternatively, you can create a client instance using the keystoneauth
+    session API. See "The novaclient Python API" page at
+    python-novaclient's doc.
+    """
+    api_version, client_class = _get_client_class_and_version(version)
+    kwargs.pop("direct_use", None)
+    return client_class(api_version=api_version, direct_use=False,
+                        *args, **kwargs)
